SystemOrganization addCategory: #ASTInterpreter!
SystemOrganization addCategory: #'ASTInterpreter-Test'!

Trait named: #TBlock
	uses: {}
	category: 'ASTInterpreter'!

!TBlock commentStamp: 'pmm 5/1/2006 12:17' prior: 0!
I define the behavoir of blocks.!

!TBlock methodsFor: 'exceptions' stamp: 'pmm 5/1/2006 12:33'!
assert
	self assert: self! !

!TBlock methodsFor: 'evaluating' stamp: 'pmm 5/1/2006 12:20'!
bench
	"See how many times I can value in 5 seconds.  I'll answer a meaningful description."

	| startTime endTime count |
	count := 0.
	endTime := Time millisecondClockValue + 5000.
	startTime := Time millisecondClockValue.
	[ Time millisecondClockValue > endTime ] whileFalse: [ self value.  count := count + 1 ].
	endTime := Time millisecondClockValue.
	^count = 1
		ifTrue: [ ((endTime - startTime) // 1000) printString, ' seconds.' ]
		ifFalse:
			[ ((count * 1000) / (endTime - startTime)) asFloat printString, ' per second.' ]! !

!TBlock methodsFor: 'private' stamp: 'pmm 5/1/2006 12:36'!
copyForSaving
	"obsolete"! !

!TBlock methodsFor: 'controlling' stamp: 'pmm 5/1/2006 12:16'!
doWhileFalse: conditionBlock
	"Evaluate the receiver once, then again as long the value of conditionBlock is false."
 
	| result |
	[result := self value.
	conditionBlock value] whileFalse.

	^ result! !

!TBlock methodsFor: 'controlling' stamp: 'pmm 5/1/2006 12:17'!
doWhileTrue: conditionBlock
	"Evaluate the receiver once, then again as long the value of conditionBlock is true."
 
	| result |
	[result := self value.
	conditionBlock value] whileTrue.

	^ result! !

!TBlock methodsFor: 'evaluating' stamp: 'pmm 5/1/2006 12:21'!
durationToRun
	"Answer the duration taken to execute this block."

	^ Duration milliSeconds: self timeToRun

! !

!TBlock methodsFor: 'exceptions' stamp: 'pmm 5/1/2006 12:33'!
ensure: aBlock
	"Evaluate a termination block after evaluating the receiver, regardless of whether the receiver's evaluation completes."

	| returnValue b |
	<primitive: 198>
	returnValue := self value.
	"aBlock wasn't nil when execution of this method began; it is nil'd out by the unwind machinery, and that's how we know it's already been evaluated ... otherwise, obviously, it needs to be evaluated"
	aBlock == nil ifFalse: [
		"nil out aBlock temp before evaluating aBlock so it is not executed again if aBlock remote returns"
		b := aBlock.
		thisContext tempAt: 1 put: nil.  "aBlock := nil"
		b value.
	].
	^ returnValue! !

!TBlock methodsFor: 'private' stamp: 'pmm 5/1/2006 12:36'!
fixTemps
	"obsolete"! !

!TBlock methodsFor: 'scheduling' stamp: 'pmm 5/1/2006 12:37'!
fork
	"Create and schedule a Process running the code in the receiver."

	^ self newProcess resume! !

!TBlock methodsFor: 'scheduling' stamp: 'pmm 5/1/2006 12:37'!
forkAndWait
	"Suspend current process while self runs"

	| semaphore |
	semaphore := Semaphore new.
	[self ensure: [semaphore signal]] fork.
	semaphore wait.
! !

!TBlock methodsFor: 'scheduling' stamp: 'pmm 5/1/2006 12:38'!
forkAt: priority 
	"Create and schedule a Process running the code in the receiver at the given priority. Answer the newly created process."

	^ self newProcess
		priority: priority;
		resume! !

!TBlock methodsFor: 'scheduling' stamp: 'pmm 5/1/2006 12:38'!
forkAt: priority named: name
	"Create and schedule a Process running the code in the receiver at the
	given priority and having the given name. Answer the newly created 
	process."

	| forkedProcess |
	forkedProcess := self newProcess.
	forkedProcess priority: priority.
	forkedProcess name: name.
	^ forkedProcess resume! !

!TBlock methodsFor: 'scheduling' stamp: 'pmm 5/1/2006 12:38'!
forkNamed: aString
	"Create and schedule a Process running the code in the receiver and
	having the given name."

	^ self newProcess name: aString; resume! !

!TBlock methodsFor: 'exceptions' stamp: 'pmm 5/1/2006 12:34'!
ifCurtailed: aBlock
	"Evaluate the receiver with an abnormal termination action."

	<primitive: 198>
	^ self value! !

!TBlock methodsFor: 'evaluating' stamp: 'pmm 5/1/2006 12:21'!
ifError: errorHandlerBlock
	"Evaluate the block represented by the receiver, and normally return it's value.  If an error occurs, the errorHandlerBlock is evaluated, and it's value is instead returned.  The errorHandlerBlock must accept zero, one, or two parameters (the error message and the receiver)."
	"Examples:
		[1 whatsUpDoc] ifError: [:err :rcvr | 'huh?'].
		[1 / 0] ifError: [:err :rcvr |
			'ZeroDivide' = err
				ifTrue: [Float infinity]
				ifFalse: [self error: err]]
"

	^ self on: Error do: [:ex |
		errorHandlerBlock valueWithPossibleArgs: {ex description. ex receiver}]! !

!TBlock methodsFor: 'accessing' stamp: 'pmm 5/1/2006 12:14'!
isBlock

	^ true! !

!TBlock methodsFor: 'scheduling' stamp: 'pmm 5/1/2006 12:38'!
newProcess
	"Answer a Process running the code in the receiver. The process is not 
	scheduled."

	<primitive: 19> "Simulation guard"
	^ Process
		forContext: 
			[self value.
			Processor terminateActive] asContext
		priority: Processor activePriority! !

!TBlock methodsFor: 'accessing' stamp: 'pmm 6/19/2006 19:05'!
numArgs
	self explicitRequirement! !

!TBlock methodsFor: 'exceptions' stamp: 'pmm 5/1/2006 12:34'!
on: exception do: handlerAction
	"Evaluate the receiver in the scope of an exception handler."

	| handlerActive |
	<primitive: 199>  "just a marker, fail and execute the following"
	handlerActive := true.
	^ self value! !

!TBlock methodsFor: 'exceptions' stamp: 'pmm 5/1/2006 12:35'!
onDNU: selector do: handleBlock
	"Catch MessageNotUnderstood exceptions but only those of the given selector (DNU stands for doesNotUnderstand:)"

	^ self on: MessageNotUnderstood do: [:exception |
		exception message selector = selector
			ifTrue: [handleBlock valueWithPossibleArgs: {exception}]
			ifFalse: [exception pass]
	  ]! !

!TBlock methodsFor: 'controlling' stamp: 'pmm 5/1/2006 12:17'!
repeat
	"Evaluate the receiver repeatedly, ending only if the block explicitly returns."

	[self value. true] whileTrue! !

!TBlock methodsFor: 'controlling' stamp: 'pmm 5/1/2006 12:18'!
repeatWithGCIf: testBlock
	| ans |
	"run the receiver, and if testBlock returns true, garbage collect and run the receiver again"
	ans := self value.
	(testBlock value: ans) ifTrue: [ Smalltalk garbageCollect. ans := self value ].
	^ans! !

!TBlock methodsFor: 'scheduling' stamp: 'pmm 5/1/2006 12:39'!
simulate
	"Like run except interpret self using Smalltalk instead of VM.  It is much slower."

	^ self newProcess simulate! !

!TBlock methodsFor: 'evaluating' stamp: 'pmm 5/1/2006 12:22'!
timeToRun
	"Answer the number of milliseconds taken to execute this block."

	^ Time millisecondsToRun: self
! !

!TBlock methodsFor: 'evaluating' stamp: 'pmm 5/1/2006 12:23'!
value
	"Evaluate the block represented by the receiver."

	^self valueWithArguments: #()
! !

!TBlock methodsFor: 'evaluating' stamp: 'pmm 5/1/2006 12:24'!
value: arg
	"Evaluate the block with the given argument. Fail if the block expects other than 1 arguments."

	^self valueWithArguments: (Array with: arg)! !

!TBlock methodsFor: 'evaluating' stamp: 'pmm 5/1/2006 12:25'!
value: arg1 value: arg2 
	"Evaluate the block with the given arguments. Fail if the block expects other than 2 arguments."

	^self valueWithArguments: (Array with: arg1 with: arg2)! !

!TBlock methodsFor: 'evaluating' stamp: 'pmm 5/1/2006 12:25'!
value: arg1 value: arg2 value: arg3 
	"Evaluate the block with the given arguments. Fail if the block expects other than 3 arguments."

	^self valueWithArguments: 
		(Array
			with: arg1
			with: arg2
			with: arg3)! !

!TBlock methodsFor: 'evaluating' stamp: 'pmm 5/1/2006 12:26'!
value: arg1 value: arg2 value: arg3 value: arg4
	"Evaluate the block with the given arguments. Fail if the block expects other than 4 arguments."

	^self valueWithArguments: 
		(Array
			with: arg1
			with: arg2
			with: arg3
			with: arg4)! !

!TBlock methodsFor: 'private' stamp: 'pmm 5/1/2006 12:36'!
valueError

	self error: 'Incompatible number of args'! !

!TBlock methodsFor: 'exceptions' stamp: 'pmm 5/1/2006 12:35'!
valueUninterruptably
	"Prevent remote returns from escaping the sender.  Even attempts to terminate (unwind) this process will be halted and the process will resume here.  A terminate message is needed for every one of these in the sender chain to get the entire process unwound."

	^ self ifCurtailed: [^ self]! !

!TBlock methodsFor: 'evaluating' stamp: 'pmm 6/19/2006 19:05'!
valueWithArguments: anArray 
	"Evaluate the block with given args. Fail if the block expects other than the given number of arguments."

	self explicitRequirement! !

!TBlock methodsFor: 'evaluating' stamp: 'pmm 5/1/2006 12:32'!
valueWithPossibleArgs: anArray
	 "Evaluate the block represented by the receiver. 
     If the block requires arguments, take them from anArray. If anArray is too
     large, the rest is ignored, if it is too small, use nil for the other arguments"

	| numArgs |
	numArgs := self numArgs.
	numArgs = 0 ifTrue: [^ self value].
	numArgs = anArray size ifTrue: [^ self valueWithArguments: anArray].
	^ self valueWithArguments: (numArgs > anArray size
		ifTrue: [anArray, (Array new: numArgs - anArray size)]
		ifFalse: [anArray copyFrom: 1 to: numArgs])! !

!TBlock methodsFor: 'evaluating' stamp: 'pmm 5/1/2006 12:31'!
valueWithPossibleArgument: anArg 

     "Evaluate the block represented by the receiver. 
     If the block requires one argument, use anArg, if it requires more than one,
     fill up the rest with nils."
	| numArgs |
	numArgs := self numArgs.
	numArgs = 0 ifTrue: [^self value].
	numArgs = 1 ifTrue: [^self value: anArg].
	numArgs  > 1 ifTrue: [^self valueWithArguments: {anArg}, (Array new: numArgs  - 1)]! !

!TBlock methodsFor: 'evaluating' stamp: 'pmm 5/1/2006 12:32'!
valueWithin: aDuration onTimeout: timeoutBlock
	"Evaluate the receiver.
	If the evaluation does not complete in less than aDuration evaluate the timeoutBlock instead"

	| theProcess delay watchdog done result |

	aDuration <= Duration zero ifTrue: [^ timeoutBlock value ].

	"the block will be executed in the current process"
	theProcess := Processor activeProcess.
	delay := aDuration asDelay.

	"make a watchdog process"
	watchdog := [
		delay wait. 	"wait for timeout or completion"
		done ifFalse: [ theProcess signalException: TimedOut ] 
	] newProcess.

	"watchdog needs to run at high priority to do its job"
	watchdog priority: Processor timingPriority.

	"catch the timeout signal"
	^ [	done := false.
		watchdog resume.				"start up the watchdog"
		result := self value.				"evaluate the receiver"
		done := true.						"it has completed, so ..."
		delay delaySemaphore signal.	"arrange for the watchdog to exit"
		result ]
			on: TimedOut do: [ :e | timeoutBlock value ].
! !

!TBlock methodsFor: 'controlling' stamp: 'pmm 5/1/2006 12:18'!
whileFalse
	"Ordinarily compiled in-line, and therefore not overridable.
	This is in case the message is sent to other than a literal block.
	Evaluate the receiver, as long as its value is false."
 
	^ [self value] whileFalse: []! !

!TBlock methodsFor: 'controlling' stamp: 'pmm 5/1/2006 12:18'!
whileFalse: aBlock 
	"Ordinarily compiled in-line, and therefore not overridable.
	This is in case the message is sent to other than a literal block.
	Evaluate the argument, aBlock, as long as the value of the receiver is false."

	^ [self value] whileFalse: [aBlock value]! !

!TBlock methodsFor: 'controlling' stamp: 'pmm 5/1/2006 12:19'!
whileTrue
	"Ordinarily compiled in-line, and therefore not overridable.
	This is in case the message is sent to other than a literal block.
	Evaluate the receiver, as long as its value is true."
 
	^ [self value] whileTrue: []! !

!TBlock methodsFor: 'controlling' stamp: 'pmm 5/1/2006 12:19'!
whileTrue: aBlock 
	"Ordinarily compiled in-line, and therefore not overridable.
	This is in case the message is sent to other than a literal block.
	Evaluate the argument, aBlock, as long as the value of the receiver is true."

	^ [self value] whileTrue: [aBlock value]! !

RBProgramNodeVisitor subclass: #PECodeInterpreter
	instanceVariableNames: 'temps'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ASTInterpreter'!

PECodeInterpreter subclass: #PEBlockInterpreter
	uses: TBlock
	instanceVariableNames: 'methodInterpreter outerInterpreter blockNode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ASTInterpreter'!

PEBlockInterpreter class
	uses: TBlock classTrait
	instanceVariableNames: ''!

!PEBlockInterpreter methodsFor: 'visitor-double dispatching' stamp: 'pmm 4/28/2006 19:01'!
acceptBlockNode: aBlockNode
	^self blockInterpreterClass
		block: aBlockNode
		methodInterpreter: methodInterpreter
		outerInterpreter: self! !

!PEBlockInterpreter methodsFor: 'exceptions' stamp: 'pmm 5/1/2006 12:33'!
assert
	self assert: self! !

!PEBlockInterpreter methodsFor: 'evaluating' stamp: 'pmm 5/1/2006 12:20'!
bench
	"See how many times I can value in 5 seconds.  I'll answer a meaningful description."

	| startTime endTime count |
	count := 0.
	endTime := Time millisecondClockValue + 5000.
	startTime := Time millisecondClockValue.
	[ Time millisecondClockValue > endTime ] whileFalse: [ self value.  count := count + 1 ].
	endTime := Time millisecondClockValue.
	^count = 1
		ifTrue: [ ((endTime - startTime) // 1000) printString, ' seconds.' ]
		ifFalse:
			[ ((count * 1000) / (endTime - startTime)) asFloat printString, ' per second.' ]! !

!PEBlockInterpreter methodsFor: 'accessing' stamp: 'pmm 4/24/2006 20:13'!
blockNode
	^blockNode! !

!PEBlockInterpreter methodsFor: 'accessing' stamp: 'pmm 4/24/2006 20:13'!
blockNode: aBlockNode 
	blockNode := aBlockNode! !

!PEBlockInterpreter methodsFor: 'accessing' stamp: 'pmm 4/27/2006 21:44'!
codeNode
	^blockNode! !

!PEBlockInterpreter methodsFor: 'private' stamp: 'pmm 5/1/2006 12:36'!
copyForSaving
	"obsolete"! !

!PEBlockInterpreter methodsFor: 'controlling' stamp: 'pmm 5/1/2006 12:16'!
doWhileFalse: conditionBlock
	"Evaluate the receiver once, then again as long the value of conditionBlock is false."
 
	| result |
	[result := self value.
	conditionBlock value] whileFalse.

	^ result! !

!PEBlockInterpreter methodsFor: 'controlling' stamp: 'pmm 5/1/2006 12:17'!
doWhileTrue: conditionBlock
	"Evaluate the receiver once, then again as long the value of conditionBlock is true."
 
	| result |
	[result := self value.
	conditionBlock value] whileTrue.

	^ result! !

!PEBlockInterpreter methodsFor: 'evaluating' stamp: 'pmm 5/1/2006 12:21'!
durationToRun
	"Answer the duration taken to execute this block."

	^ Duration milliSeconds: self timeToRun

! !

!PEBlockInterpreter methodsFor: 'exceptions' stamp: 'pmm 5/1/2006 12:33'!
ensure: aBlock
	"Evaluate a termination block after evaluating the receiver, regardless of whether the receiver's evaluation completes."

	| returnValue b |
	<primitive: 198>
	returnValue := self value.
	"aBlock wasn't nil when execution of this method began; it is nil'd out by the unwind machinery, and that's how we know it's already been evaluated ... otherwise, obviously, it needs to be evaluated"
	aBlock == nil ifFalse: [
		"nil out aBlock temp before evaluating aBlock so it is not executed again if aBlock remote returns"
		b := aBlock.
		thisContext tempAt: 1 put: nil.  "aBlock := nil"
		b value.
	].
	^ returnValue! !

!PEBlockInterpreter methodsFor: 'private' stamp: 'pmm 5/1/2006 12:36'!
fixTemps
	"obsolete"! !

!PEBlockInterpreter methodsFor: 'scheduling' stamp: 'pmm 5/1/2006 12:37'!
fork
	"Create and schedule a Process running the code in the receiver."

	^ self newProcess resume! !

!PEBlockInterpreter methodsFor: 'scheduling' stamp: 'pmm 5/1/2006 12:37'!
forkAndWait
	"Suspend current process while self runs"

	| semaphore |
	semaphore := Semaphore new.
	[self ensure: [semaphore signal]] fork.
	semaphore wait.
! !

!PEBlockInterpreter methodsFor: 'scheduling' stamp: 'pmm 5/1/2006 12:38'!
forkAt: priority 
	"Create and schedule a Process running the code in the receiver at the given priority. Answer the newly created process."

	^ self newProcess
		priority: priority;
		resume! !

!PEBlockInterpreter methodsFor: 'scheduling' stamp: 'pmm 5/1/2006 12:38'!
forkAt: priority named: name
	"Create and schedule a Process running the code in the receiver at the
	given priority and having the given name. Answer the newly created 
	process."

	| forkedProcess |
	forkedProcess := self newProcess.
	forkedProcess priority: priority.
	forkedProcess name: name.
	^ forkedProcess resume! !

!PEBlockInterpreter methodsFor: 'scheduling' stamp: 'pmm 5/1/2006 12:38'!
forkNamed: aString
	"Create and schedule a Process running the code in the receiver and
	having the given name."

	^ self newProcess name: aString; resume! !

!PEBlockInterpreter methodsFor: 'exceptions' stamp: 'pmm 5/1/2006 15:57'!
ifCurtailed: aBlock
	"Evaluate the receiver with an abnormal termination action."

	^ [ self value ] ifCurtailed: aBlock! !

!PEBlockInterpreter methodsFor: 'evaluating' stamp: 'pmm 5/1/2006 12:21'!
ifError: errorHandlerBlock
	"Evaluate the block represented by the receiver, and normally return it's value.  If an error occurs, the errorHandlerBlock is evaluated, and it's value is instead returned.  The errorHandlerBlock must accept zero, one, or two parameters (the error message and the receiver)."
	"Examples:
		[1 whatsUpDoc] ifError: [:err :rcvr | 'huh?'].
		[1 / 0] ifError: [:err :rcvr |
			'ZeroDivide' = err
				ifTrue: [Float infinity]
				ifFalse: [self error: err]]
"

	^ self on: Error do: [:ex |
		errorHandlerBlock valueWithPossibleArgs: {ex description. ex receiver}]! !

!PEBlockInterpreter methodsFor: 'accessing-variables' stamp: 'pmm 4/28/2006 10:28'!
instanceVariableAt: aString
	^methodInterpreter instanceVariableAt: aString! !

!PEBlockInterpreter methodsFor: 'accessing-variables' stamp: 'pmm 4/28/2006 10:28'!
instanceVariableAt: aString put: anObject
	^methodInterpreter instanceVariableAt: aString put: anObject! !

!PEBlockInterpreter methodsFor: 'accessing' stamp: 'pmm 5/1/2006 12:14'!
isBlock

	^ true! !

!PEBlockInterpreter methodsFor: 'accessing' stamp: 'pmm 4/24/2006 20:54'!
methodClass
	^methodInterpreter methodClass! !

!PEBlockInterpreter methodsFor: 'accessing' stamp: 'pmm 4/24/2006 19:55'!
methodInterpreter
	^methodInterpreter! !

!PEBlockInterpreter methodsFor: 'accessing' stamp: 'pmm 4/24/2006 20:14'!
methodInterpreter: aMethodInterpreter 
	methodInterpreter := aMethodInterpreter! !

!PEBlockInterpreter methodsFor: 'scheduling' stamp: 'pmm 5/1/2006 12:38'!
newProcess
	"Answer a Process running the code in the receiver. The process is not 
	scheduled."

	<primitive: 19> "Simulation guard"
	^ Process
		forContext: 
			[self value.
			Processor terminateActive] asContext
		priority: Processor activePriority! !

!PEBlockInterpreter methodsFor: 'accessing' stamp: 'pmm 8/23/2006 13:54'!
nodesForInstrumentation
	^Array with: self blockNode copy! !

!PEBlockInterpreter methodsFor: 'accessing' stamp: 'pmm 4/28/2006 10:53'!
numArgs
	^blockNode numArgs! !

!PEBlockInterpreter methodsFor: 'exceptions' stamp: 'pmm 5/1/2006 15:56'!
on: exception do: handlerAction
	"Evaluate the receiver in the scope of an exception handler."

	^[ self value ] on: exception do: handlerAction! !

!PEBlockInterpreter methodsFor: 'exceptions' stamp: 'pmm 5/1/2006 12:35'!
onDNU: selector do: handleBlock
	"Catch MessageNotUnderstood exceptions but only those of the given selector (DNU stands for doesNotUnderstand:)"

	^ self on: MessageNotUnderstood do: [:exception |
		exception message selector = selector
			ifTrue: [handleBlock valueWithPossibleArgs: {exception}]
			ifFalse: [exception pass]
	  ]! !

!PEBlockInterpreter methodsFor: 'accessing' stamp: 'pmm 4/28/2006 10:42'!
outerInterpreter
	^outerInterpreter! !

!PEBlockInterpreter methodsFor: 'accessing' stamp: 'pmm 4/28/2006 10:43'!
outerInterpreter: aCodeInterpreter 
	outerInterpreter := aCodeInterpreter! !

!PEBlockInterpreter methodsFor: 'accessing' stamp: 'pmm 4/24/2006 20:51'!
receiver
	^methodInterpreter receiver! !

!PEBlockInterpreter methodsFor: 'controlling' stamp: 'pmm 5/1/2006 12:17'!
repeat
	"Evaluate the receiver repeatedly, ending only if the block explicitly returns."

	[self value. true] whileTrue! !

!PEBlockInterpreter methodsFor: 'controlling' stamp: 'pmm 5/1/2006 12:18'!
repeatWithGCIf: testBlock
	| ans |
	"run the receiver, and if testBlock returns true, garbage collect and run the receiver again"
	ans := self value.
	(testBlock value: ans) ifTrue: [ Smalltalk garbageCollect. ans := self value ].
	^ans! !

!PEBlockInterpreter methodsFor: 'returning' stamp: 'pmm 4/26/2006 17:45'!
return: anObject
	self methodInterpreter return: anObject! !

!PEBlockInterpreter methodsFor: 'scheduling' stamp: 'pmm 5/1/2006 12:39'!
simulate
	"Like run except interpret self using Smalltalk instead of VM.  It is much slower."

	^ self newProcess simulate! !

!PEBlockInterpreter methodsFor: 'accessing-variables' stamp: 'pmm 4/28/2006 10:52'!
temporaryVariableAt: aString
	^temps
		at: aString
		ifAbsent: [
			outerInterpreter temporaryVariableAt: aString ]! !

!PEBlockInterpreter methodsFor: 'accessing-variables' stamp: 'pmm 4/28/2006 10:46'!
temporaryVariableAt: aString put: anObject
	^(temps includesKey: aString)
		ifTrue: [ temps at: aString put: anObject ]
		ifFalse: [ outerInterpreter temporaryVariableAt: aString put: anObject ]! !

!PEBlockInterpreter methodsFor: 'evaluating' stamp: 'pmm 5/1/2006 12:22'!
timeToRun
	"Answer the number of milliseconds taken to execute this block."

	^ Time millisecondsToRun: self
! !

!PEBlockInterpreter methodsFor: 'evaluating' stamp: 'pmm 5/1/2006 12:23'!
value
	"Evaluate the block represented by the receiver."

	^self valueWithArguments: #()
! !

!PEBlockInterpreter methodsFor: 'evaluating' stamp: 'pmm 5/1/2006 12:24'!
value: arg
	"Evaluate the block with the given argument. Fail if the block expects other than 1 arguments."

	^self valueWithArguments: (Array with: arg)! !

!PEBlockInterpreter methodsFor: 'evaluating' stamp: 'pmm 5/1/2006 12:25'!
value: arg1 value: arg2 
	"Evaluate the block with the given arguments. Fail if the block expects other than 2 arguments."

	^self valueWithArguments: (Array with: arg1 with: arg2)! !

!PEBlockInterpreter methodsFor: 'evaluating' stamp: 'pmm 5/1/2006 12:25'!
value: arg1 value: arg2 value: arg3 
	"Evaluate the block with the given arguments. Fail if the block expects other than 3 arguments."

	^self valueWithArguments: 
		(Array
			with: arg1
			with: arg2
			with: arg3)! !

!PEBlockInterpreter methodsFor: 'evaluating' stamp: 'pmm 5/1/2006 12:26'!
value: arg1 value: arg2 value: arg3 value: arg4
	"Evaluate the block with the given arguments. Fail if the block expects other than 4 arguments."

	^self valueWithArguments: 
		(Array
			with: arg1
			with: arg2
			with: arg3
			with: arg4)! !

!PEBlockInterpreter methodsFor: 'private' stamp: 'pmm 5/1/2006 12:36'!
valueError

	self error: 'Incompatible number of args'! !

!PEBlockInterpreter methodsFor: 'exceptions' stamp: 'pmm 5/1/2006 12:35'!
valueUninterruptably
	"Prevent remote returns from escaping the sender.  Even attempts to terminate (unwind) this process will be halted and the process will resume here.  A terminate message is needed for every one of these in the sender chain to get the entire process unwound."

	^ self ifCurtailed: [^ self]! !

!PEBlockInterpreter methodsFor: 'evaluating' stamp: 'pmm 4/27/2006 21:45'!
valueWithArguments: anArray
	self initializeArgumentsWith: anArray.
	^self visitNode: blockNode body! !

!PEBlockInterpreter methodsFor: 'evaluating' stamp: 'pmm 5/1/2006 12:32'!
valueWithPossibleArgs: anArray
	 "Evaluate the block represented by the receiver. 
     If the block requires arguments, take them from anArray. If anArray is too
     large, the rest is ignored, if it is too small, use nil for the other arguments"

	| numArgs |
	numArgs := self numArgs.
	numArgs = 0 ifTrue: [^ self value].
	numArgs = anArray size ifTrue: [^ self valueWithArguments: anArray].
	^ self valueWithArguments: (numArgs > anArray size
		ifTrue: [anArray, (Array new: numArgs - anArray size)]
		ifFalse: [anArray copyFrom: 1 to: numArgs])! !

!PEBlockInterpreter methodsFor: 'evaluating' stamp: 'pmm 5/1/2006 12:31'!
valueWithPossibleArgument: anArg 

     "Evaluate the block represented by the receiver. 
     If the block requires one argument, use anArg, if it requires more than one,
     fill up the rest with nils."
	| numArgs |
	numArgs := self numArgs.
	numArgs = 0 ifTrue: [^self value].
	numArgs = 1 ifTrue: [^self value: anArg].
	numArgs  > 1 ifTrue: [^self valueWithArguments: {anArg}, (Array new: numArgs  - 1)]! !

!PEBlockInterpreter methodsFor: 'evaluating' stamp: 'pmm 5/1/2006 12:32'!
valueWithin: aDuration onTimeout: timeoutBlock
	"Evaluate the receiver.
	If the evaluation does not complete in less than aDuration evaluate the timeoutBlock instead"

	| theProcess delay watchdog done result |

	aDuration <= Duration zero ifTrue: [^ timeoutBlock value ].

	"the block will be executed in the current process"
	theProcess := Processor activeProcess.
	delay := aDuration asDelay.

	"make a watchdog process"
	watchdog := [
		delay wait. 	"wait for timeout or completion"
		done ifFalse: [ theProcess signalException: TimedOut ] 
	] newProcess.

	"watchdog needs to run at high priority to do its job"
	watchdog priority: Processor timingPriority.

	"catch the timeout signal"
	^ [	done := false.
		watchdog resume.				"start up the watchdog"
		result := self value.				"evaluate the receiver"
		done := true.						"it has completed, so ..."
		delay delaySemaphore signal.	"arrange for the watchdog to exit"
		result ]
			on: TimedOut do: [ :e | timeoutBlock value ].
! !

!PEBlockInterpreter methodsFor: 'controlling' stamp: 'pmm 5/1/2006 12:18'!
whileFalse
	"Ordinarily compiled in-line, and therefore not overridable.
	This is in case the message is sent to other than a literal block.
	Evaluate the receiver, as long as its value is false."
 
	^ [self value] whileFalse: []! !

!PEBlockInterpreter methodsFor: 'controlling' stamp: 'pmm 5/1/2006 12:18'!
whileFalse: aBlock 
	"Ordinarily compiled in-line, and therefore not overridable.
	This is in case the message is sent to other than a literal block.
	Evaluate the argument, aBlock, as long as the value of the receiver is false."

	^ [self value] whileFalse: [aBlock value]! !

!PEBlockInterpreter methodsFor: 'controlling' stamp: 'pmm 5/1/2006 12:19'!
whileTrue
	"Ordinarily compiled in-line, and therefore not overridable.
	This is in case the message is sent to other than a literal block.
	Evaluate the receiver, as long as its value is true."
 
	^ [self value] whileTrue: []! !

!PEBlockInterpreter methodsFor: 'controlling' stamp: 'pmm 5/1/2006 12:19'!
whileTrue: aBlock 
	"Ordinarily compiled in-line, and therefore not overridable.
	This is in case the message is sent to other than a literal block.
	Evaluate the argument, aBlock, as long as the value of the receiver is true."

	^ [self value] whileTrue: [aBlock value]! !

!PECodeInterpreter methodsFor: 'visitor-double dispatching' stamp: 'pmm 4/24/2006 20:12'!
acceptArrayNode: anArrayNode 
	^anArrayNode children collect: [ :each |
		self visitNode: each ]! !

!PECodeInterpreter methodsFor: 'visitor-double dispatching' stamp: 'pmm 4/26/2006 17:06'!
acceptAssignmentNode: anAssignmentNode 
	self
		set: anAssignmentNode variable
		toValue: (self visitNode: anAssignmentNode value)! !

!PECodeInterpreter methodsFor: 'visitor-double dispatching' stamp: 'pmm 4/28/2006 10:45'!
acceptBlockNode: aBlockNode
	self subclassResponsibility! !

!PECodeInterpreter methodsFor: 'visitor-double dispatching' stamp: 'pmm 4/24/2006 20:31'!
acceptCascadeNode: aCascadeNode
	| receiver last |
	receiver := self visitNode: aCascadeNode receiver.
	aCascadeNode messages do: [ :each |
		last := receiver
			perform: each selector
			withArguments: (each arguments collect: [ :argument |
				self visitNode: argument] ) ].
	^last! !

!PECodeInterpreter methodsFor: 'visitor-double dispatching' stamp: 'pmm 4/29/2006 11:58'!
acceptDoItNode: aDoItNode
	| localTempNames |
	localTempNames := aDoItNode tempNames.
	temps := Dictionary new: (localTempNames size).
	localTempNames do: [ :each |
		temps at: each put: nil ].
	self visitNode: aDoItNode body
	! !

!PECodeInterpreter methodsFor: 'visitor-double dispatching' stamp: 'pmm 4/24/2006 20:08'!
acceptLiteralNode: aLiteralNode
	^aLiteralNode value
! !

!PECodeInterpreter methodsFor: 'visitor-double dispatching' stamp: 'md 4/16/2007 21:25'!
acceptMessageNode: aMessageNode
	| isSuperSend messageReceiver messageArguments |
	isSuperSend := aMessageNode isSuperSend.
	messageReceiver := isSuperSend
		ifTrue: [ self receiver ]
		ifFalse: [ self visitNode: aMessageNode receiver ].
	messageArguments := aMessageNode arguments collect: [ :each |
		self visitNode: each ].
	^isSuperSend
		ifTrue: [
			messageReceiver
				perform: aMessageNode selector
				withArguments: messageArguments
				inSuperclass: self methodClass superclass ]
		ifFalse: [
			messageReceiver
				perform: aMessageNode selector
				withArguments: messageArguments ]! !

!PECodeInterpreter methodsFor: 'visitor-double dispatching' stamp: 'pmm 4/27/2006 23:37'!
acceptReturnNode: aReturnNode 
	self return: (self visitNode: aReturnNode value)! !

!PECodeInterpreter methodsFor: 'visitor-double dispatching' stamp: 'pmm 4/24/2006 20:22'!
acceptSequenceNode: aSequenceNode 
	| last |
	last := nil.
	aSequenceNode statements do: [:each |
		last := self visitNode: each ].
	^last! !

!PECodeInterpreter methodsFor: 'visitor-double dispatching' stamp: 'md 3/31/2007 19:01'!
acceptVariableNode: aVariableNode
	| name |
	name := aVariableNode name.
	(#('self' 'super') includes: name)
		ifTrue: [ ^self receiver ].
	name = 'true' ifTrue: [ ^true ].
	name = 'false' ifTrue: [ ^false ].
	name = 'nil' ifTrue: [ ^nil ].
	name = 'thisContext' ifTrue: [ ^ PEInterpreterContext codeInterpreter: self ].
	
	^aVariableNode
		ifTemp: [ self temporaryVariableAt: name ]
		ifInstance: [ self instanceVariableAt: name ]
		ifGlobal: [ (self receiver class bindingOf: name) value ]! !

!PECodeInterpreter methodsFor: 'private' stamp: 'md 3/31/2007 19:17'!
blockInterpreterClass
	^PEBlockInterpreter! !

!PECodeInterpreter methodsFor: 'accessing' stamp: 'pmm 4/27/2006 21:44'!
codeNode
	self subclassResponsibility ! !

!PECodeInterpreter methodsFor: 'initialize-release' stamp: 'pmm 4/29/2006 12:45'!
initializeArgumentsWith: anArray
	| localTempNames |
	localTempNames := self codeNode tempNames.
	temps := Dictionary new: (localTempNames size).
	localTempNames do: [ :each |
		temps at: each put: nil ].
	self codeNode arguments withIndexDo: [ :each :index | 
		temps at: each name put: (anArray at: index) ].! !

!PECodeInterpreter methodsFor: 'accessing-variables' stamp: 'pmm 4/26/2006 10:52'!
instanceVariableAt: aString
	self subclassResponsibility ! !

!PECodeInterpreter methodsFor: 'accessing-variables' stamp: 'pmm 4/26/2006 17:03'!
instanceVariableAt: aString put: anObject
	self subclassResponsibility ! !

!PECodeInterpreter methodsFor: 'accessing' stamp: 'pmm 4/24/2006 20:52'!
methodClass
	self subclassResponsibility ! !

!PECodeInterpreter methodsFor: 'accessing' stamp: 'pmm 4/24/2006 20:16'!
methodInterpreter
	self subclassResponsibility ! !

!PECodeInterpreter methodsFor: 'accessing' stamp: 'pmm 4/24/2006 20:51'!
receiver
	self subclassResponsibility ! !

!PECodeInterpreter methodsFor: 'returning' stamp: 'pmm 4/24/2006 19:55'!
return: anObject
	self subclassResponsibility ! !

!PECodeInterpreter methodsFor: 'accessing-variables' stamp: 'pmm 7/1/2006 15:21'!
set: aVariable toValue: anObject
	aVariable
		ifTemp: [ self temporaryVariableAt: aVariable name put: anObject ]
		ifInstance: [ self instanceVariableAt: aVariable name put: anObject ]
		ifGlobal: [ (self receiver class bindingOf: aVariable name) value: anObject ].
	^anObject! !

!PECodeInterpreter methodsFor: 'accessing-variables' stamp: 'pmm 4/26/2006 10:53'!
temporaryVariableAt: aString
	self subclassResponsibility ! !

!PECodeInterpreter methodsFor: 'accessing-variables' stamp: 'pmm 4/26/2006 17:02'!
temporaryVariableAt: aString put: anObject
	self subclassResponsibility ! !

!PECodeInterpreter methodsFor: 'accessing' stamp: 'pmm 4/24/2006 16:29'!
temps
	^temps! !

!PECodeInterpreter methodsFor: 'accessing' stamp: 'pmm 4/24/2006 16:30'!
temps: aDictionary 
	temps := aDictionary! !

PECodeInterpreter subclass: #PEMethodInterpreter
	instanceVariableNames: 'receiver methodNode instanceVariableMap escaper'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ASTInterpreter'!

!PEMethodInterpreter methodsFor: 'visitor-double dispatching' stamp: 'pmm 4/28/2006 19:01'!
acceptBlockNode: aBlockNode
	^self blockInterpreterClass
		block: aBlockNode
		interpreter: self! !

!PEMethodInterpreter methodsFor: 'visitor-double dispatching' stamp: 'pmm 7/3/2006 09:11'!
acceptMethodNode: aMethodNode 
	aMethodNode isPrimitive
		ifTrue: [ self executePrimitve: aMethodNode ]
		ifFalse: [ self interpret: aMethodNode ]! !

!PEMethodInterpreter methodsFor: 'accessing' stamp: 'pmm 7/3/2006 08:36'!
arguments
	^self codeNode arguments
		collect: [ :each | temps at: each name ]! !

!PEMethodInterpreter methodsFor: 'accessing' stamp: 'pmm 4/27/2006 21:44'!
codeNode
	^methodNode! !

!PEMethodInterpreter methodsFor: 'executing' stamp: 'md 3/31/2007 19:18'!
executePrimitve: aMethodNode
	| newMethodNode newBody newMethod |
	newMethodNode := aMethodNode copy.
	newBody := RBSequenceNode statement: (
		RBMessageNode
			receiver: (PEObjectLiteralNode value: self)
			selector: #interpret:
			argument: (PEObjectLiteralNode value: aMethodNode)).
	newMethodNode body: newBody.
	newMethod := (ASTTranslator new
		visitNode: newMethodNode;
		ir) compiledMethodWith: #(0 0 0 0).
	self return: (newMethod valueWithReceiver: self receiver arguments: self arguments)! !

!PEMethodInterpreter methodsFor: 'accessing-variables' stamp: 'pmm 4/26/2006 11:23'!
instanceVariableAt: aString
	^self receiver instVarAt: (self instanceVariableMap at: aString)! !

!PEMethodInterpreter methodsFor: 'accessing-variables' stamp: 'pmm 4/26/2006 17:20'!
instanceVariableAt: aString put: anObject
	^self receiver
		instVarAt: (self instanceVariableMap at: aString)
		 put: anObject! !

!PEMethodInterpreter methodsFor: 'accessing' stamp: 'pmm 4/27/2006 23:05'!
instanceVariableMap
	instanceVariableMap isNil ifTrue: [
		instanceVariableMap := Dictionary new.
		receiver class allInstVarNames withIndexDo: [ :each :index |
			instanceVariableMap at: each put: index ] ].
	^instanceVariableMap! !

!PEMethodInterpreter methodsFor: 'executing' stamp: 'pmm 7/3/2006 08:21'!
interpret: aMethodNode
	self visitNode: aMethodNode body.
	self returnSelf! !

!PEMethodInterpreter methodsFor: 'accessing' stamp: 'pmm 4/24/2006 20:54'!
methodClass
	^methodNode methodClass! !

!PEMethodInterpreter methodsFor: 'accessing' stamp: 'pmm 4/24/2006 20:16'!
methodInterpreter
	^self! !

!PEMethodInterpreter methodsFor: 'accessing' stamp: 'pmm 4/27/2006 22:04'!
methodNode: aMethodNode 
	methodNode := aMethodNode! !

!PEMethodInterpreter methodsFor: 'accessing' stamp: 'pmm 4/24/2006 20:20'!
receiver
	^receiver! !

!PEMethodInterpreter methodsFor: 'accessing' stamp: 'pmm 4/24/2006 20:20'!
receiver: anObject
	receiver := anObject! !

!PEMethodInterpreter methodsFor: 'returning' stamp: 'pmm 4/27/2006 23:43'!
return: anObject
	escaper value: anObject! !

!PEMethodInterpreter methodsFor: 'returning' stamp: 'pmm 4/27/2006 23:43'!
returnSelf
	self return: receiver ! !

!PEMethodInterpreter methodsFor: 'executing' stamp: 'pmm 4/27/2006 22:06'!
run
	^self withEscaper: [
		self visitNode: methodNode ]! !

!PEMethodInterpreter methodsFor: 'accessing-variables' stamp: 'pmm 4/28/2006 10:53'!
temporaryVariableAt: aString
	^temps at: aString! !

!PEMethodInterpreter methodsFor: 'accessing-variables' stamp: 'pmm 4/26/2006 17:22'!
temporaryVariableAt: aString put: anObject
	^temps
		at: aString
		put: anObject! !

!PEMethodInterpreter methodsFor: 'visiting' stamp: 'pmm 4/24/2006 16:39'!
visitBlockArguments: aNodeCollection
	self shouldNotImplement! !

!PEMethodInterpreter methodsFor: 'escaping' stamp: 'pmm 4/26/2006 21:13'!
withEscaper: aBlock
	| old |
	old := escaper.
	escaper := [ :value | ^ value ].
	^ aBlock ensure: [ escaper := old ]! !

Object subclass: #PEInterpretationExamples
	instanceVariableNames: 'instVar'
	classVariableNames: 'ClassVar'
	poolDictionaries: ''
	category: 'ASTInterpreter-Test'!

!PEInterpretationExamples methodsFor: 'accessing' stamp: 'pmm 5/31/2006 10:55'!
classVar
	^ClassVar! !

!PEInterpretationExamples methodsFor: 'accessing' stamp: 'pmm 5/31/2006 10:55'!
classVar: anObject
	ClassVar := anObject! !

!PEInterpretationExamples methodsFor: 'examples' stamp: 'pmm 4/26/2006 21:41'!
exampleArgument: anObject
	^anObject + 5! !

!PEInterpretationExamples methodsFor: 'examples' stamp: 'pmm 4/26/2006 21:49'!
exampleArray
	^{ 1 . 1 + 1 . 1 + 2 . 2 + 3  }! !

!PEInterpretationExamples methodsFor: 'examples' stamp: 'pmm 4/26/2006 21:40'!
exampleArrayLiteral
	^#(1 2 3 4)! !

!PEInterpretationExamples methodsFor: 'examples' stamp: 'pmm 4/26/2006 21:45'!
exampleBlockReturn
	#(1 2 3) do: [ :each |
		each = 2 ifTrue: [ ^each ] ]! !

!PEInterpretationExamples methodsFor: 'examples' stamp: 'pmm 4/26/2006 21:44'!
exampleBlockTemp
	instVar := 0.
	#(1 2 3) do: [ :each |
		| x |
		x := each.
		x := x * x.
		instVar := instVar + x ].
	^instVar! !

!PEInterpretationExamples methodsFor: 'examples' stamp: 'pmm 4/28/2006 20:40'!
exampleBranch
	^1 > 2
		ifTrue: [ 1000 + 337 ]
		ifFalse: [ 30000 + 1000 + 337 ]! !

!PEInterpretationExamples methodsFor: 'examples' stamp: 'pmm 4/26/2006 21:48'!
exampleCascade
	^OrderedCollection new
		add: 1;
		add: 2;
		add: 3;
		yourself! !

!PEInterpretationExamples methodsFor: 'examples' stamp: 'pmm 4/26/2006 21:42'!
exampleDo
	| x |
	x := 0.
	#(1 2 3) do: [ :each |
		x := x + each ].
	^x! !

!PEInterpretationExamples methodsFor: 'examples' stamp: 'pmm 5/1/2006 16:21'!
exampleEnsure
	| x |
	x := ValueHolder new
		contents: 0
		yourself.
	[ 0 / 1 ]
		ensure: [ x contents: 20 ].
	^x contents! !

!PEInterpretationExamples methodsFor: 'examples' stamp: 'md 4/16/2007 21:26'!
exampleGlobal
	^PEInterpretationExamples name! !

!PEInterpretationExamples methodsFor: 'examples' stamp: 'pmm 4/26/2006 21:43'!
exampleInstVar
	instVar := 0.
	#(1 2 3) do: [ :each |
		instVar := instVar + each ].
	^instVar! !

!PEInterpretationExamples methodsFor: 'examples' stamp: 'pmm 5/1/2006 16:08'!
exampleOnDo
	| x |
	x := 0.
	[ 1 / 0 ]
		on: ZeroDivide
		do: [ x:= 20 ].
	^x! !

!PEInterpretationExamples methodsFor: 'examples' stamp: 'pmm 4/28/2006 18:59'!
exampleOuterBlock
	| x |
	x := 0.
	#(1 2 3) do: [ :each |
		| blockTemp |
		blockTemp := each.
		blockTemp = 2 ifTrue: [
			blockTemp := blockTemp + 10 ].
		x := x + blockTemp ].
	^x! !

!PEInterpretationExamples methodsFor: 'examples' stamp: 'pmm 4/26/2006 21:40'!
exampleReturnSelf
	1 + 2! !

!PEInterpretationExamples methodsFor: 'examples' stamp: 'pmm 4/26/2006 21:41'!
exampleTemp
	| temp |
	temp := 1000.
	temp := temp + 337.
	^temp! !

!PEInterpretationExamples methodsFor: 'examples' stamp: 'pmm 4/26/2006 21:55'!
exampleTrue
	^true! !

!PEInterpretationExamples methodsFor: 'examples' stamp: 'pmm 5/1/2006 08:16'!
exampleWhile
	| x |
	x := 40.
	[ (x > 20) ]
		whileTrue: [ x := x - 2 ].
	^x! !

!PEInterpretationExamples methodsFor: 'examples' stamp: 'md 4/16/2007 21:30'!
printString
	^super printString , ' RLY'! !

Object subclass: #PEInterpreterContext
	instanceVariableNames: 'codeInterpreter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ASTInterpreter'!

!PEInterpreterContext methodsFor: 'accessing' stamp: 'pmm 4/26/2006 09:02'!
codeInterpreter
	^codeInterpreter! !

!PEInterpreterContext methodsFor: 'accessing' stamp: 'pmm 4/26/2006 09:02'!
codeInterpreter: aCodeInterpreter 
	codeInterpreter := aCodeInterpreter! !

TestCase subclass: #PEInterpretationTest
	instanceVariableNames: 'example'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ASTInterpreter-Test'!

!PEInterpretationTest methodsFor: 'private' stamp: 'pmm 4/27/2006 23:14'!
interpret: aSymbol
	^self interpret: aSymbol withArguments: #()! !

!PEInterpretationTest methodsFor: 'private' stamp: 'pmm 4/27/2006 23:14'!
interpret: aSymbol with: anObject
	^self interpret: aSymbol withArguments: (Array with: anObject)! !

!PEInterpretationTest methodsFor: 'private' stamp: 'md 4/16/2007 21:17'!
interpret: aSymbol withArguments: aCollection
	^(self reflectiveMethod: aSymbol)
		interpret: aSymbol
		with: aCollection
		in: example! !

!PEInterpretationTest methodsFor: 'private' stamp: 'md 4/16/2007 21:17'!
reflectiveMethod: aSymbol
	^(example class >> aSymbol) reflectiveMethod! !

!PEInterpretationTest methodsFor: 'running' stamp: 'md 4/16/2007 21:21'!
setUp
	super setUp.
	example := PEInterpretationExamples new ! !

!PEInterpretationTest methodsFor: 'testing' stamp: 'pmm 7/1/2006 15:25'!
testClassVar
	"self run: #testClassVar"
	(self interpret: #classVar: with: 666).
	self assert: (self interpret: #classVar) = 666.
	
	(self interpret: #classVar: with: 'Nova Cat').
	self assert: (self interpret: #classVar) = 'Nova Cat'.
	
	! !

!PEInterpretationTest methodsFor: 'testing' stamp: 'pmm 4/27/2006 22:53'!
testExampleArgument
	"self run: #testExampleArgument"
	self assert: (self interpret: #exampleArgument: with: 10) = 15! !

!PEInterpretationTest methodsFor: 'testing' stamp: 'pmm 4/26/2006 21:52'!
testExampleArray
	self assert: (self interpret: #exampleArray) = #(1 2 3 5)! !

!PEInterpretationTest methodsFor: 'testing' stamp: 'pmm 4/26/2006 21:52'!
testExampleArrayLiteral
	self assert: (self interpret: #exampleArrayLiteral) = #(1 2 3 4)! !

!PEInterpretationTest methodsFor: 'testing' stamp: 'pmm 4/26/2006 21:53'!
testExampleBlockReturn
	self assert: (self interpret: #exampleBlockReturn) = 2! !

!PEInterpretationTest methodsFor: 'testing' stamp: 'pmm 4/26/2006 21:54'!
testExampleBlockTemp
	self assert: (self interpret: #exampleBlockTemp) = 14! !

!PEInterpretationTest methodsFor: 'testing' stamp: 'pmm 4/28/2006 20:37'!
testExampleBranch
	self assert: (self interpret: #exampleBranch) = 31337! !

!PEInterpretationTest methodsFor: 'testing' stamp: 'pmm 4/26/2006 21:54'!
testExampleCascade
	self assert: (self interpret: #exampleCascade) = #(1 2 3) asOrderedCollection! !

!PEInterpretationTest methodsFor: 'testing' stamp: 'pmm 4/26/2006 21:55'!
testExampleDo
	self assert: (self interpret: #exampleDo) = 6! !

!PEInterpretationTest methodsFor: 'testing' stamp: 'pmm 5/1/2006 16:06'!
testExampleEnsure
	self assert: (self interpret: #exampleEnsure) = 20! !

!PEInterpretationTest methodsFor: 'testing' stamp: 'md 4/16/2007 21:26'!
testExampleGlobal
	self assert: (self interpret: #exampleGlobal) = 'PEInterpretationExamples'! !

!PEInterpretationTest methodsFor: 'testing' stamp: 'pmm 4/26/2006 21:56'!
testExampleInstVar
	self assert: (self interpret: #exampleInstVar) = 6! !

!PEInterpretationTest methodsFor: 'testing' stamp: 'pmm 5/1/2006 16:06'!
testExampleOnDo
	self assert: (self interpret: #exampleOnDo) = 20! !

!PEInterpretationTest methodsFor: 'testing' stamp: 'pmm 4/28/2006 19:00'!
testExampleOuterBlock
	self assert: (self interpret: #exampleOuterBlock) = 16! !

!PEInterpretationTest methodsFor: 'testing' stamp: 'pmm 4/26/2006 21:57'!
testExampleReturnSelf
	self assert: (self interpret: #exampleReturnSelf) == example! !

!PEInterpretationTest methodsFor: 'testing' stamp: 'md 4/16/2007 21:27'!
testExampleTemp
	self assert: (self interpret: #printString) =  'a PEInterpretationExamples RLY'! !

!PEInterpretationTest methodsFor: 'testing' stamp: 'pmm 4/26/2006 21:55'!
testExampleTrue
	self assert: (self interpret: #exampleTrue)! !

!PEInterpretationTest methodsFor: 'testing' stamp: 'pmm 5/1/2006 08:16'!
testExampleWhile
	self assert: (self interpret: #exampleWhile) = 20! !

!PEInterpretationTest methodsFor: 'testing' stamp: 'md 4/16/2007 21:11'!
testPrimitive
	| methodNode result method |
	methodNode := PESqueakParser parseMethod: (SmalltalkImage >> #getSystemAttribute:) getSource.
	methodNode methodClass: SmalltalkImage.
	method := ReflectiveMethod new
		methodNode: methodNode;
		yourself.
	method beInterpreted.
	result := method run: #getSystemAttribute: with: #(1001) in: SmalltalkImage current.
	self deny: result isNil.
	self assert: result = (SmalltalkImage current getSystemAttribute: 1001)! !
